{********************************************************************}
{                     I R Q U T I L  . P A S                         }
{*------------------------------------------------------------------*}
{ Aufgabe        : Stellt Funktionen fr Hardware-Interrupt          }
{                  Programmierung zur Verfgung.                     }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 12.03.1994                                        }
{ letztes Update :  8.10.1994                                        }
{********************************************************************}
Unit IRQUTIL;

Interface

Const
{- IRQ-Controller Portadressen --------------------------------------}
 MASTER_PIC      =    $20;             { Basisadresse des Master-PIC }
 SLAVE_PIC       =    $A0;              { Basisadresse des Slave-PIC }
 IRQ_MASK        =    $01;               { Offset zu Maskierungsport }

{- IRQ-Kommandos ----------------------------------------------------}
 EOI             =    $20;       { Unspezifizierter End of Interrupt }

 MASTER_FIRST_VECTOR = $08;        { Software-Vektoren der Hardware- }
 SLAVE_FIRST_VECTOR  = $70;        { Interrupts                      }

{-- Prototypen -------------}

Procedure irq_Enable    ( iIRQ : Integer );
Procedure irq_Disable   ( iIRQ : Integer );
Procedure irq_SendEOI   ( iIRQ : Integer );
Function  irq_SetHandler( iIRQ : Integer; lpHandler : Pointer ) : Pointer;
Function  irq_ReadMask  ( iController : Integer ) : Byte;
Function  irq_ReadISR   ( iController : Integer ) : Byte;
Function  irq_ReadIRR   ( iController : Integer ) : Byte;

Implementation
Uses DOS;

{********************************************************************}
{ irq_Enable : Hardware-Interrupt zulassen                           }
{*------------------------------------------------------------------*}
{ Eingabe : iIRQ - Nummer des Hardware-Interrupts (0-15), der zu-    }
{                  gelassen werden soll                              }
{*------------------------------------------------------------------*}
{ Info : Das An- und Abschalten von IRQ's geschieht durch            }
{        bermittlung eines OCW1 (Operation Control Word) genannten  }
{        Bytes an den Basisport des IRQ-Controllers.                 }
{********************************************************************}
Procedure irq_Enable( iIRQ : Integer );

var iPort : Integer;

Begin
  {-- zunchst Port Adresse des zustndigen PIC's ermitteln ---------}
  { ( 0-7 = MASTER_PIC , 8-15 = SLAVE_PIC )                          }
  if iIRQ <= 7 then iPort := MASTER_PIC
               else iPort := SLAVE_PIC ;
  iPort := iPort + IRQ_MASK;            { Maskierungs-Port auswhlen }

  iIRQ := iIRQ and $0007;      { PIC-Interruptnummer ermitteln (0-7) }
                               { Bit Lschen -> Interrupt zugelassen }
  port[iPort] := port[iPort] and not ( 1 shl iIRQ );
End;

{********************************************************************}
{ irq_Disable : Hardware-Interrupt unterdrcken                      }
{*------------------------------------------------------------------*}
{ Eingabe : iIRQ - Nummer des Hardware-Interrupts (0-15) der ver-    }
{                  boten werden soll                                 }
{********************************************************************}
Procedure irq_Disable( iIRQ : Integer );

var iPort : Integer;
Begin                 
  {-- zunchst Port Adresse des zustndigen PIC's ermitteln ---------}
  { ( 0-7 = MASTER_PIC , 8-15 = SLAVE_PIC )                          }
  if iIRQ <= 7 then iPort := MASTER_PIC
               else iPort := SLAVE_PIC ;

  iPort := iPort + IRQ_MASK;            { Maskierungs-Port auswhlen }

  iIRQ := iIRQ and $0007;      { PIC-Interruptnummer ermitteln (0-7) }
                                  { Bit Setzen -> Interrupt gesperrt }
  port[iPort] := port[iPort] or ( 1 shl iIRQ );
End;

{********************************************************************}
{ irq_SendEOI : "End Of Interrupt" signalisieren                     }
{*------------------------------------------------------------------*}
{ Eingabe : iIRQ : Nummer des Hardware-Interrupts (0-15) der komplet }
{                  abgearbeitet wurde.                               }
{ Info : - Ein Interrupt kann erst dann wieder ausgelst werden,     }
{          nachdem der vorherig vollstndig abgearbeitet wurde.      }
{          wurde. Um den PIC's zu signalisieren, da ein Interrupt-  }
{          Handler beendet wurde, mu an die PICs ein EOI-Kommando   }
{          gesendet werden.                                          }
{********************************************************************}
Procedure irq_SendEOI( iIRQ : Integer );

Begin                    
  {-- Bei IRQ 8 - 15 auch Slave benachrichtigen --}
  if  iIRQ > 7 then port[ SLAVE_PIC] := EOI;
  port[ MASTER_PIC] := EOI;   { Dem MASTER immer EOI signalisieren }
End;

{********************************************************************}
{ irq_SetHandler : Neuen Interrupt-Handler installieren              }
{*------------------------------------------------------------------*}
{ Eingabe : iIRQ      - Nummer des Interrupts, der neuen Handler     }
{                       erhalten soll.                               }
{           lpHandler - Adresse des neuen Handlers                   }
{ Ausgabe : Adresse des alten Handlers.                              }
{********************************************************************}
Function irq_SetHandler( iIRQ : Integer; lpHandler : Pointer) : Pointer;

var lpOldHandler : Pointer;
    iVect        : Integer;

Begin
  {-- Interupt-Vektor des Hardware-Interrupts ermitteln ------}
  { IRQ 0 - 7  = Vektoren $08 - $0F                                  }
  { IRQ 8 - 15 = Vektoren $70 - $77                                  }
  if  iIRQ <= 7 then
    iVect := ( MASTER_FIRST_VECTOR + iIRQ )
  else
    iVect := ( SLAVE_FIRST_VECTOR + ( iIRQ and $7 ) );

  irq_Disable( iIRQ );   { Hardware- und Software-Interrupts sperren }
  asm cli end;

  GetIntVec( iVect,lpOldhandler );            { Alten Handler retten }
  SetIntVec( iVect,lpHandler );               { Neuen Handler setzen }

  asm sti end;                        { Software-Interrupts erlauben }

  {- Falls ein Handler bergeben wurde auch wieder entsprechenden --}
  {- Hardware-Interrupt erlauben                                  --}
  if lpHandler <> NIL then
    irq_Enable( iIRQ );
                            
  irq_SetHandler := lpOldHandler; {Adr. des alten Handlers zurckg.}
End;

{********************************************************************}
{ irq_ReadMask : Lese Maskierungen eines IRQ-Controllers             }
{*------------------------------------------------------------------*}
{ Eingabe : iController - Basisport des IRQ-Controllers ($20/$A0)    }
{ Ausgabe : Maskierung der vom IRQ-Controller bedienten IRQ's.       }
{           Gesetzte Bits : IRQ wird nicht ausgelst.                }
{*------------------------------------------------------------------*}
{ Info : Die Maskierungen eines PIC's knnen durch einfaches aus-    }
{        lesen des Mask-Registers ermittelt werden. ($21/$A1)        }
{********************************************************************}
Function irq_ReadMask( iController : Integer ) : Byte;

Begin
  irq_ReadMask := port[iController + IRQ_MASK];
End;

{********************************************************************}
{ irq_ReadISR : Lese Statusregister eines Irq-Controllers            }
{*------------------------------------------------------------------*}
{ Eingabe : iController - Basisport des IRQ-Controllers ($20/$A0)    }
{ Ausgabe : Statusregister der vom IRQ-Controller bedienten IRQ's.   }
{           Gesetzte Bits : die enstprechende IRQ-Routine wird       }
{           augenblich ausgefhrt oder durch einen hher prio-       }
{           risierten IRQ unterbrochen.                              }
{*------------------------------------------------------------------*}
{ Info : Bevor der IRQ-Controller den Inhalt des ISR-Registers       }
{        ausgibt, mu das OCW3 genannte Byte an den Controller       }
{        bermittelt werden.                                         }
{********************************************************************}
Function irq_ReadISR( iController : Integer ) : Byte;

Begin           
  {-- OCW3: $0B = Keine Operation , kein Poll, ISR lesen ------------}
  port[iController] := $0B;
  irq_ReadISR := port[iController ];
End;

{********************************************************************}
{ irq_ReadIRR : Lese Requestregister eines Irq-Controllers           }
{*------------------------------------------------------------------*}
{ Eingabe : iController - Basisport des IRQ-Controllers ($20/$A0)    }
{ Ausgabe : Requestregister der vom IRQ-Controller bedienten IRQ's.  }
{           Gesetzte Bits : die enstprechende IRQ-Routine soll als   }
{           nchstes ausgefhrt werden, sofer kein hher prio-       }
{           risierter Request ansteht.                               }
{*------------------------------------------------------------------*}
{ Info : Bevor der IRQ-Controller den Inhalt des ISR-Registers       }
{        mu das OCW3 genannte Byte an den Controller bermittelt    }
{        werden.                                                     }
{********************************************************************}
Function irq_ReadIRR( iController : Integer ) : Byte;
Begin           
  {-- OCW3: $0A = Keine Operation , kein Poll, IRR lesen ------------}
  port[iController] := $0A ;
  irq_ReadIRR := port[iController];
End;

End.